import concurrent
import logging
import os
import time
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime, timedelta
from typing import List

from pymongo import MongoClient

from sais.autotrain.dataservice.config.const import LOGGER_NAME
from sais.autotrain.dataservice.model.data_model import Coordinate

logger = logging.getLogger(LOGGER_NAME)
logging.basicConfig(level=logging.INFO)

MONGO_URL = f'mongodb://{os.getenv("MONGO_USER", "root")}:{os.getenv("MONGO_PASSWORD", "yourPassword")}@{os.getenv("MONGO_HOST", "localhost")}:{os.getenv("MONGO_PORT", 8087)}'


def query_nwp_mongo(src: str, start_time: str, period_interval: int, period: int, start_hour: int, end_hour: int,
              forecast_interval: int, coords: list[Coordinate], vars: list[str], workers: int):
    query_start_time = time.time()  # 记录查询开始时间
    # 解析请求参数
    start_time = datetime.strptime(start_time, "%y%m%d%H")
    # 所有起报时间点
    forcast_start_times = []
    # 所有预报时间点
    forcast_times = []
    # 预报步长
    steps = []
    # forcast_time 按月分组
    month_groups = {}
    # 发布时次
    for i in range(period):
        forcast_start_time = start_time + timedelta(hours=i * period_interval)
        # logger.info(f'forcast_start_time: {forcast_start_time}')
        forcast_start_time_str = forcast_start_time.strftime("%Y-%m-%d %H:%M:%S")
        forcast_start_times.append(forcast_start_time_str)
        # 记录到月分组中
        month_key = forcast_start_time.strftime('%Y%m')
        if month_key not in month_groups:
            month_groups[month_key] = []
        month_groups[month_key].append(forcast_start_time_str)

        for hour in range(start_hour, end_hour + 1, forecast_interval):
            current_step = timedelta(hours=hour)
            forcast_time = forcast_start_time + current_step
            # logger.info(f'forcast_time: {forcast_time}')
            forcast_times.append(forcast_time.strftime("%Y-%m-%d %H:%M:%S"))

            # 格式化时间步长
            formatted_step = f"P{current_step.days}DT{current_step.seconds // 3600}H{(current_step.seconds // 60) % 60}M{current_step.seconds % 60}S"
            steps.append(formatted_step)
            # logger.info(f'forcast_start_times: {forcast_start_times}')
        # logger.info(f'forcast_times: {forcast_times}')
    result = query_mongo_multi_thread(workers, src, month_groups, steps, vars, coords)
    all_end_time = time.time()  # 记录查询结束时间
    all_execution_time = all_end_time - query_start_time
    logger.info(f"总耗时: {all_execution_time} 秒, 总数量：{len(result) if result else 0}")
    return result


def run_query(db: MongoClient, src: str, month_item: dict, forcast_times: list[str], steps: list[str], vars: list[str],
              coords: List[Coordinate]):
    query_conditions = []

    # 四舍五入经纬度到小数点后一位
    rounded_coords = [
        {"req_lat": round(coord.req_lat, 1), "req_lon": round(coord.req_lon, 1)}
        for coord in coords
    ]
    query_conditions.append({
        "src": src,
        "time": {"$in": forcast_times},
        "latitude": {"$in": [coord["req_lat"] for coord in rounded_coords]},
        "longitude": {"$in": [coord["req_lon"] for coord in rounded_coords]},
        "step": {"$in": steps},
        "$and": [{var: {"$exists": True}} for var in vars]
    })
    collection = get_collection(db, src, month_item)
    vars_dict = {key: 1 for key in vars}
    query = {"$and": query_conditions}
    logger.info(f'query: {query_conditions}')
    query_results = collection.find(query, {
        "src": 1,
        "time": 1,
        "valid_time": 1,
        "step": 1,
        **vars_dict
    }).sort({
        "time": 1,
        "step": 1
    })
    return list(query_results)


def query_mongo_multi_thread(workers, src: str, month_group: dict, steps: list[str], vars: list[str], coords: List[Coordinate]):
    # 并发查询
    all_results = []
    client = MongoClient(MONGO_URL,
                         serverSelectionTimeoutMS=6000,
                         connectTimeoutMS=6000)
    db = client[os.getenv("MONGO_DB", "auto_train")]
    with ThreadPoolExecutor(max_workers=workers) as executor:
        future_to_month = {
            executor.submit(run_query, db, src, month_key, month_group[month_key], steps, vars, coords): month_key
            for month_key in month_group
        }
        for future in concurrent.futures.as_completed(future_to_month):
            month_key = future_to_month[future]
            try:
                results = future.result()
                all_results.extend(results)
                # 对所有结果进行排序
                all_results.sort(key=lambda x: (x['time'], x['valid_time']))
            except Exception as exc:
                print(f'Error for month {month_key}: {exc}')

    return all_results


def get_collection(db, src, year_month):
    cname = f"nwp_{src.split('/')[0]}_{year_month}"
    logger.info(f'collection name: {cname}')
    return db[cname]


def get_year_month(dt):
    year_month = dt.strftime("%Y%m")
    return year_month



import json
import logging
import os
import time

import requests

from sais.autotrain.dataservice.auth.auth_info import EnvVarCredentialsProvider
from sais.autotrain.dataservice.client.mongo_client import query_nwp_mongo
from sais.autotrain.dataservice.config.const import LOGGER_NAME, ENDPOINT
from sais.autotrain.dataservice.model.data_model import Task, StatusEnum, QueryNWPRequest, QueryStationRequest, \
    QueryObservationRequest, BaseRsp

logger = logging.getLogger(LOGGER_NAME)
logging.basicConfig(level=logging.INFO)


class QueryNWPHandler(object):
    def __init__(self, endpoint=os.getenv("ENDPOINT", ENDPOINT), auth_provider: EnvVarCredentialsProvider = None):
        self.endpoint = endpoint
        self.auth_provider = auth_provider

    def submit_query_task(self, query: QueryNWPRequest):
        """提交查询任务并返回任务ID"""
        url = f"{self.endpoint}/api/v1/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            logger.info(f"Query task submitted successfully. Task ID: {query_result_json.get('data').get('task_id')}")
            return query_result_json.get('data').get('task_id')
        else:
            logger.error(f"Failed to submit query task. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query task.")

    def query_task_status(self, task_id):
        """通过任务ID查询任务状态"""
        url = f"{self.endpoint}/api/v1/query/{task_id}"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}"
        }
        response = requests.get(url, headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            logger.info(f"Query task status: {query_result_json.get('data')}")
            return Task(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to query task status. Response: {query_result_json}")
            raise ValueError(f"Failed to query task status.{query_result_json.get('msg')}")

    def wait_for_task_completion(self, task_id, poll_interval=5):
        """
        轮询查询任务状态直到任务完成
        状态：
            - PENDING: 等待执行
            - RUNNING: 执行中
            - FINISHED: 完成（已成功生成json文件）
            - FAILED: 失败
        """
        while True:
            task = self.query_task_status(task_id)
            if task.status == StatusEnum.FINISHED:
                if os.path.exists(task.file_path):
                    with open(task.file_path, 'r') as file:
                        return json.load(file)
                else:
                    return None
            elif task.status == StatusEnum.FAILED:
                raise Exception(f"Query task failed. Task ID: {task.task_id}")
            time.sleep(poll_interval)  # 轮询间隔

    def execute_query_mongo(self, query_params: QueryNWPRequest):
        return query_nwp_mongo(query_params.src, query_params.start_time, query_params.period_interval,
                               query_params.period,
                               query_params.start_hour,
                               query_params.end_hour,
                               query_params.forecast_interval,
                               query_params.coords,
                               query_params.vars,
                               query_params.workers)

    def execute_query(self, query_params: QueryNWPRequest):
        if query_params.engine == "mongo":
            result = self.execute_query_mongo(query_params)
            return result
        else:
            """封装的查询功能函数，提交查询任务并轮询直到返回任务状态结果"""
            task_id = self.submit_query_task(query_params)
            print(f"Query task submitted with task ID: {task_id}")
            return self.wait_for_task_completion(task_id)


class QueryStationRelatedHandler(object):
    def __init__(self, endpoint=ENDPOINT, auth_provider: EnvVarCredentialsProvider = None):
        self.endpoint = endpoint
        self.auth_provider = auth_provider

    def execute_query_station(self, query_params):
        """封装的查询功能函数，提交查询任务并轮询直到返回任务状态结果"""
        station_data = self.submit_query_station(query_params)
        return station_data

    def execute_query_observation(self, query_params):
        """封装的查询功能函数，提交查询任务并轮询直到返回任务状态结果"""
        observation_data = self.submit_query_observation(query_params)
        return observation_data

    def submit_query_observation(self, query: QueryObservationRequest):
        """提交观测查询"""
        url = f"{self.endpoint}/api/v1/observation/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            logger.info(f"Query observation submitted successfully.")
            return BaseRsp(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to submit query observation. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query observation.{query_result_json.get('msg')}")

    def submit_query_station(self, query: QueryStationRequest):
        """提交场站查询"""
        url = f"{self.endpoint}/api/v1/station/query"
        headers = {
            # "Authorization": f"Bearer {self.auth_provider.token}",
            "Content-Type": "application/json"
        }
        response = requests.post(url, json=query.model_dump(by_alias=True, exclude_none=True), headers=headers)
        query_result_json = response.json()
        if query_result_json.get('success'):
            logger.info(f"Query station submitted successfully.")
            return BaseRsp(**query_result_json.get('data'))
        else:
            logger.error(f"Failed to submit query station. Response: {query_result_json}")
            raise ValueError(f"Failed to submit query station.{query_result_json.get('msg')}")
